"""
Texas module
============

This module presents two classes to handle the way state holidays are managed
in Texas.

The :class:`TexasBase` class gathers all available holidays for Texas,
according to this document:
http://www.statutes.legis.state.tx.us/Docs/GV/htm/GV.662.htm

The :class:`Texas` class includes all national and state holidays, as described
in the said document. This should be the "default" Texas calendar class, to be
used in most cases.

But if state holidays are supposed to be observed by most of the workforces,
any employee can chose to skip one of these days and replace it by another.

If at some point you need to create a specific calendar class based on Texas
calendar, you can either use the :class:`TexasBase` class or directly the
:class:`Texas` class and overwrite/override the :method:`get_fixed_holidays()`
and/or :method:`get_variable_days()` to fit your needs.

Example:

.. code::

    class TexasCustom(TexasBase):
        # This will include the confederate heroes day
        texas_include_confederate_heroes = True

        FIXED_HOLIDAYS = TexasBase.FIXED_HOLIDAYS + (
            (7, 14, "Bastille Day!"),
        )

        def get_variable_days(self, year):
            days = super().get_variable_days(year)
            days.append(
                (self.get_nth_weekday_in_month(year, 1, 15), "Special Day")
            )
            return days

"""
from datetime import date

from ..registry_tools import iso_register
from .core import UnitedStates


class TexasBase(UnitedStates):
    """Texas Base (w/o State holidays)"""
    include_columbus_day = False
    texas_include_confederate_heroes = False
    texas_include_independance_day = False
    texas_san_jacinto_day = False
    texas_emancipation_day = False
    texas_lyndon_johnson_day = False
    # Non-Texas-specific state holidays
    include_thanksgiving_friday = False
    include_christmas_eve = False
    include_boxing_day = False

    def get_fixed_holidays(self, year):
        days = super().get_fixed_holidays(year)
        if self.texas_include_confederate_heroes:
            days.append(
                (date(year, 1, 19), "Confederate Heroes Day")
            )

        if self.texas_include_independance_day:
            days.append(
                (date(year, 3, 2), "Texas Independence Day")
            )

        if self.texas_san_jacinto_day:
            days.append(
                (date(year, 4, 21), "San Jacinto Day")
            )

        if self.texas_emancipation_day:
            days.append(
                (date(year, 6, 19), "Emancipation Day in Texas"),
            )

        if self.texas_lyndon_johnson_day:
            days.append(
                (date(year, 8, 27), "Lyndon B. Jonhson Day"),
            )
        return days


@iso_register('US-TX')
class Texas(TexasBase):
    """Texas"""
    texas_include_confederate_heroes = True
    texas_include_independance_day = True
    texas_san_jacinto_day = True
    texas_emancipation_day = True
    texas_lyndon_johnson_day = True
    include_thanksgiving_friday = True
    include_christmas_eve = True
    include_boxing_day = True



from .core import UnitedStates
from ..registry_tools import iso_register


@iso_register('US-MA')
class Massachusetts(UnitedStates):
    """Massachusetts"""
    include_patriots_day = True


class SuffolkCountyMassachusetts(Massachusetts):
    """Suffolk County, Massachusetts"""
    FIXED_HOLIDAYS = UnitedStates.FIXED_HOLIDAYS + (
        (3, 17, 'Evacuation Day'),
        (6, 17, 'Bunker Hill Day'),
    )



import warnings
from datetime import date
from unittest import TestCase

from ..core import Calendar


class CoreCalendarTest(TestCase):
    cal_class = Calendar

    def setUp(self):
        super().setUp()
        self.year = date.today().year
        self.cal = self.cal_class()


class GenericCalendarTest(CoreCalendarTest):
    test_include_january_1st = True

    def setUp(self):
        super().setUp()
        warnings.simplefilter('ignore')

    def test_weekend_days(self):
        class_name = self.cal_class.__name__
        if class_name in ('Calendar',):
            return
        try:
            self.cal.get_weekend_days()
        except NotImplementedError:
            assert False, (self.cal, class_name)

    def test_january_1st(self):
        class_name = self.cal_class.__name__
        if class_name in ('Calendar',):
            return
        holidays = self.cal.holidays_set(self.year)
        if self.test_include_january_1st:
            self.assertIn(date(self.year, 1, 1), holidays)
        else:
            self.assertNotIn(date(self.year, 1, 1), holidays)



from ..core import WesternCalendar
from ..registry_tools import iso_register


@iso_register('HU')
class Hungary(WesternCalendar):
    'Hungary'

    include_easter_sunday = True
    include_easter_monday = True
    include_whit_sunday = True
    whit_sunday_label = "Pentecost Sunday"
    include_whit_monday = True
    whit_monday_label = "Pentecost Monday"
    include_boxing_day = True
    boxing_day_label = "Second Day of Christmas"
    include_all_saints = True

    FIXED_HOLIDAYS = WesternCalendar.FIXED_HOLIDAYS + (
        (3, 15, "National Day"),
        (5, 1, "Labour Day"),
        (8, 20, "St Stephen's Day"),
        (10, 23, "National Day"),
    )

    def get_variable_days(self, year):
        # As of 2017, Good Friday became a holiday
        self.include_good_friday = (year >= 2017)
        days = super().get_variable_days(year)
        return days



from datetime import date

from ..astronomy import calculate_equinoxes, solar_term


def test_calculate_some_equinoxes():
    assert calculate_equinoxes(2010) == (date(2010, 3, 20), date(2010, 9, 23))
    assert calculate_equinoxes(2010, 'Asia/Taipei') == (
        date(2010, 3, 21), date(2010, 9, 23)
    )
    assert calculate_equinoxes(2013) == (date(2013, 3, 20), date(2013, 9, 22))
    assert calculate_equinoxes(2014) == (date(2014, 3, 20), date(2014, 9, 23))
    assert calculate_equinoxes(2020) == (date(2020, 3, 20), date(2020, 9, 22))


def test_qingming_festivals():
    assert solar_term(2001, 15) == date(2001, 4, 4)
    assert solar_term(2001, 15, 'Asia/Taipei') == date(2001, 4, 5)
    assert solar_term(2011, 15) == date(2011, 4, 5)
    assert solar_term(2014, 15) == date(2014, 4, 4)
    assert solar_term(2016, 15) == date(2016, 4, 4)
    assert solar_term(2017, 15) == date(2017, 4, 4)


def test_qingming_festivals_hk():
    assert solar_term(2018, 15, 'Asia/Hong_Kong') == date(2018, 4, 5)
    assert solar_term(2019, 15, 'Asia/Hong_Kong') == date(2019, 4, 5)
    assert solar_term(2020, 15, 'Asia/Hong_Kong') == date(2020, 4, 4)
    assert solar_term(2021, 15, 'Asia/Hong_Kong') == date(2021, 4, 4)



from datetime import date
from ..core import WesternCalendar, THU, MON
from ..registry_tools import iso_register


@iso_register('IS')
class Iceland(WesternCalendar):
    'Iceland'

    include_holy_thursday = True
    include_good_friday = True
    include_easter_monday = True
    include_ascension = True
    include_whit_monday = True
    include_christmas_eve = True
    include_boxing_day = True
    boxing_day_label = "St Stephen's Day"

    FIXED_HOLIDAYS = WesternCalendar.FIXED_HOLIDAYS + (
        (5, 1, "Labour Day"),
        (6, 17, "Icelandic National Day"),
        (12, 31, "New Year's Eve"),
    )

    def get_first_day_of_summer(self, year):
        """It's the first thursday *after* April, 18th.
        If April the 18th is a thursday, then it jumps to the 24th.
        """
        return Iceland.get_nth_weekday_in_month(
            year, 4, THU,
            start=date(year, 4, 19))

    def get_commerce_day(self, year):
        return Iceland.get_nth_weekday_in_month(year, 8, MON)

    def get_variable_days(self, year):
        days = super().get_variable_days(year)
        days.extend([
            (self.get_first_day_of_summer(year), "First day of summer"),
            (self.get_commerce_day(year), "Commerce Day"),
        ])
        return days



from datetime import date

from .core import UnitedStates
from ..registry_tools import iso_register


@iso_register('US-AS')
class AmericanSamoa(UnitedStates):
    "American Samoa"
    include_boxing_day = True
    boxing_day_label = "Family Day"

    def get_flag_day(self, year):
        """
        Flag day is on April 17th
        """
        return (
            date(year, 4, 17),
            "Flag Day"
        )

    def get_variable_days(self, year):
        days = super().get_variable_days(year)
        days.extend([
            self.get_flag_day(year),
        ])
        return days



from .core import UnitedStates
from ..registry_tools import iso_register


@iso_register('US-ME')
class Maine(UnitedStates):
    """Maine"""
    include_thanksgiving_friday = True
    include_patriots_day = True



from ..core import IslamoWesternCalendar, SAT, SUN
from ..registry_tools import iso_register


@iso_register('BJ')
class Benin(IslamoWesternCalendar):
    "Benin"
    # Christian holidays
    include_easter_monday = True
    include_ascension = True
    include_whit_monday = True
    include_assumption = True
    include_all_saints = True
    # Islamic holidays
    include_prophet_birthday = True
    include_eid_al_fitr = True
    include_day_of_sacrifice = True
    include_day_of_sacrifice_label = "Tabaski"

    FIXED_HOLIDAYS = IslamoWesternCalendar.FIXED_HOLIDAYS + (
        (1, 10, "Traditional Day"),
        (5, 1, "Labour Day"),
        (8, 1, "Independence Day"),
        (10, 26, "Armed Forces Day"),
        (11, 30, "National Day"),
    )

    # Explicitly assign these WE days, Benin has adopted the western workweek
    WEEKEND_DAYS = (SAT, SUN)



from datetime import date

from ..core import WesternCalendar
from ..registry_tools import iso_register


@iso_register('HR')
class Croatia(WesternCalendar):
    'Croatia'

    FIXED_HOLIDAYS = WesternCalendar.FIXED_HOLIDAYS + (
        (5, 1, "International Workers' Day"),
        (6, 22, "Anti-Fascist Struggle Day"),
        (8, 5, "Victory & Homeland Thanksgiving & Day of Croatian defenders"),
    )

    include_epiphany = True
    include_easter_sunday = True
    include_easter_monday = True
    include_corpus_christi = True
    include_assumption = True
    include_all_saints = True
    include_christmas = True
    include_boxing_day = True
    boxing_day_label = "St. Stephen's Day"

    def get_fixed_holidays(self, year):
        days = super().get_fixed_holidays(year)

        if year >= 2020:
            days.extend([
                # Statehood date has changed in 2020
                (date(year, 5, 30), "Statehood Day"),
                # Remembrance Day has been added in 2020
                (date(year, 11, 18), "Remembrance Day"),
            ])
        else:
            days.extend([
                (date(year, 6, 25), "Statehood Day"),
                # Independance day was a holiday until 2020
                (date(year, 10, 8), "Independence Day"),
            ])

        return days



from datetime import date
from unittest import TestCase

from . import GenericCalendarTest
from ..america import (
    Brazil, BrazilSaoPauloState,
    BrazilAcre, BrazilAlagoas, BrazilAmapa, BrazilAmazonas, BrazilBahia,
    BrazilCeara, BrazilDistritoFederal, BrazilEspiritoSanto, BrazilGoias,
    BrazilMaranhao, BrazilMinasGerais, BrazilMatoGrosso, BrazilMatoGrossoDoSul,
    BrazilPara, BrazilParaiba, BrazilPernambuco, BrazilPiaui, BrazilParana,
    BrazilRioDeJaneiro, BrazilRioGrandeDoNorte, BrazilRioGrandeDoSul,
    BrazilRondonia, BrazilRoraima, BrazilSantaCatarina, BrazilSergipe,
    BrazilTocantins,
    # Cities
    BrazilSaoPauloCity, BrazilVitoriaCity, BrazilVilaVelhaCity,
    BrazilCariacicaCity, BrazilGuarapariCity, BrazilSerraCity,
    BrazilRioBrancoCity, BrazilMaceioCity, BrazilManausCity, BrazilMacapaCity,
    BrazilSalvadorCity, BrazilFortalezaCity, BrazilGoianiaCity,
    BrazilBeloHorizonteCity, BrazilCampoGrandeCity, BrazilCuiabaCity,
    BrazilBelemCity, BrazilJoaoPessoaCity, BrazilRecifeCity,
    BrazilTeresinaCity, BrazilCuritibaCity, BrazilNatalCity,
    BrazilPortoVelhoCity, BrazilBoaVistaCity, BrazilPortoAlegreCity,
    BrazilChapecoCity, BrazilFlorianopolisCity, BrazilJoinvilleCity,
    BrazilAracajuCity, BrazilSorocabaCity, BrazilPalmasCity,
    # Banks
    BrazilBankCalendar,
)
from ..america.brazil import IBGE_REGISTER, IBGE_TUPLE


class BrazilTest(GenericCalendarTest):
    cal_class = Brazil
    test_include_consciencia_negra = False

    def test_year_2013_federal(self):
        holidays = self.cal.holidays_set(2013)
        self.assertIn(date(2013, 1, 1), holidays)  # new year
        self.assertIn(date(2013, 4, 21), holidays)  # Tiradentes
        self.assertIn(date(2013, 5, 1), holidays)  # Dia do trabalhador
        self.assertIn(date(2013, 9, 7), holidays)  # Dia da Independência
        self.assertIn(date(2013, 10, 12), holidays)  # Nossa Senhora Aparecida
        self.assertIn(date(2013, 11, 2), holidays)  # Finados
        self.assertIn(date(2013, 11, 15), holidays)  # Proclamação da República
        self.assertIn(date(2013, 12, 25), holidays)  # Natal

    def test_consciencia_negra(self):
        # Consciência Negra day is not a national holiday
        # It's triggered in the appropriate classes, so this test needs to
        # be overwritten.
        month, day = self.cal.consciencia_negra_day
        consciencia_negra_day = date(self.year, month, day)
        holidays = self.cal.holidays_set(self.year)
        if self.test_include_consciencia_negra:
            # Included where needed
            self.assertIn(consciencia_negra_day, holidays)
        else:
            # By default, not in the holidays.
            self.assertNotIn(consciencia_negra_day, holidays)


class BrazilAcreTest(BrazilTest):
    cal_class = BrazilAcre

    def test_year_2017_state(self):
        holidays = self.cal.holidays_set(2017)
        self.assertIn(date(2017, 1, 23), holidays)  # Dia do evangélico
        self.assertIn(date(2017, 6, 15), holidays)  # niversário do Acre
        self.assertIn(date(2017, 9, 5), holidays)  # Dia da Amazônia
        # Assinatura do Tratado de Petrópolis
        self.assertIn(date(2017, 11, 17), holidays)
        # Início da Revolução Acreana
        self.assertIn(date(2017, 8, 6), holidays)


class BrazilAlagoasTest(BrazilTest):
    cal_class = BrazilAlagoas
    test_include_consciencia_negra = True

    def test_year_2017_state(self):
        holidays = self.cal.holidays_set(2017)
        self.assertIn(date(2017, 6, 24), holidays)  # São João
        self.assertIn(date(2017, 6, 29), holidays)  # São Pedro
        # Emancipação política de Alagoas
        self.assertIn(date(2017, 9, 16), holidays)
        self.assertIn(date(2017, 11, 20), holidays)  # Consciência Negra


class BrazilAmapaTest(BrazilTest):
    cal_class = BrazilAmapa
    test_include_consciencia_negra = True

    def test_year_2017_state(self):
        holidays = self.cal.holidays_set(2017)
        # Dia de São José
        sao_jose = date(2017, 3, 19)
        self.assertIn(sao_jose, holidays)
        # Check its label
        self.assertEqual(
            self.cal.get_holiday_label(sao_jose), "Dia de São José")

        self.assertIn(date(2017, 7, 25), holidays)  # São Tiago
        self.assertIn(date(2017, 10, 5), holidays)  # Criação do estado
        self.assertIn(date(2017, 11, 20), holidays)  # Consciência Negra
        self.assertIn(date(2017, 9, 13), holidays)  # Aniversário da Amapá


class BrazilAmazonasTest(BrazilTest):
    cal_class = BrazilAmazonas
    test_include_consciencia_negra = True

    def test_year_2017_state(self):
        holidays = self.cal.holidays_set(2017)
        # Elevação do Amazonas á categoria de província
        self.assertIn(date(2017, 9, 5), holidays)
        self.assertIn(date(2017, 11, 20), holidays)  # Consciência Negra
        # Dia de Nossa Senhora da Conceição
        self.assertIn(date(2017, 12, 8), holidays)


class BrazilBahiaTest(BrazilTest):
    cal_class = BrazilBahia

    def test_year_2017_state(self):
        holidays = self.cal.holidays_set(2017)
        self.assertIn(date(2017, 7, 2), holidays)  # Independência da Bahia


class BrazilCearaTest(BrazilTest):
    cal_class = BrazilCeara

    def test_year_2017_state(self):
        holidays = self.cal.holidays_set(2017)
        self.assertIn(date(2017, 3, 19), holidays)  # São José
        self.assertIn(date(2017, 3, 23), holidays)  # Data Manga do Ceará
        self.assertIn(date(2017, 3, 25), holidays)  # Aniversário do Ceará


class BrazilDistritoFederalTest(BrazilTest):
    cal_class = BrazilDistritoFederal

    def test_year_2017_state(self):
        holidays = self.cal.holidays_set(2017)
        self.assertIn(date(2017, 4, 21), holidays)  # Fundação de Brasília
        self.assertIn(date(2017, 11, 30), holidays)  # Dia do Evangélico


class BrazilEspiritoSantoTest(BrazilTest):
    cal_class = BrazilEspiritoSanto

    def test_year_2017_state(self):
        holidays = self.cal.holidays_set(2017)
        self.assertIn(date(2017, 10, 28), holidays)  # Dia do Servidor Público


class BrazilGoiasTest(BrazilTest):
    cal_class = BrazilGoias

    def test_year_2017_state(self):
        holidays = self.cal.holidays_set(2017)
        self.assertIn(date(2017, 10, 28), holidays)  # Dia do Servidor Público


class BrazilMaranhaoTest(BrazilTest):
    cal_class = BrazilMaranhao

    def test_year_2017_state(self):
        holidays = self.cal.holidays_set(2017)
        # Adesão do Maranhão á independência do Brasil
        self.assertIn(date(2017, 7, 28), holidays)
        # Dia de Nossa Senhora da Conceição
        self.assertIn(date(2017, 12, 8), holidays)


class BrazilMinasGeraisTest(BrazilTest):
    cal_class = BrazilMinasGerais

    def test_year_2017_state(self):
        holidays = self.cal.holidays_set(2017)
        # Aniversário de Minas Geraisl
        self.assertIn(date(2017, 4, 21), holidays)


class BrazilMatoGrossoTest(BrazilTest):
    cal_class = BrazilMatoGrosso
    test_include_consciencia_negra = True

    def test_year_2017_state(self):
        holidays = self.cal.holidays_set(2017)
        self.assertIn(date(2017, 11, 29), holidays)  # Consciência Negra


class BrazilMatoGrossoDoSulTest(BrazilTest):
    cal_class = BrazilMatoGrossoDoSul

    def test_year_2017_state(self):
        holidays = self.cal.holidays_set(2017)
        self.assertIn(date(2017, 10, 11), holidays)  # Criação do estado


class BrazilParaTest(BrazilTest):
    cal_class = BrazilPara

    def test_year_2017_state(self):
        holidays = self.cal.holidays_set(2017)
        # Adesão do Grão-Pará á independência do Brasil
        self.assertIn(date(2017, 8, 15), holidays)


class BrazilParaibaTest(BrazilTest):
    cal_class = BrazilParaiba

    def test_year_2017_state(self):
        holidays = self.cal.holidays_set(2017)
        self.assertIn(date(2017, 8, 5), holidays)  # Fundação do Estado
        # Homenagem à memória do ex-presidente João Pessoa
        self.assertIn(date(2017, 7, 26), holidays)


class BrazilPernambucoTest(BrazilTest):
    cal_class = BrazilPernambuco

    def test_year_2017_state(self):
        holidays = self.cal.holidays_set(2017)
        self.assertIn(date(2017, 6, 24), holidays)  # São João


class BrazilPiauiTest(BrazilTest):
    cal_class = BrazilPiaui

    def test_year_2017_state(self):
        holidays = self.cal.holidays_set(2017)
        # Dia da Batalha do Jenipapo
        self.assertIn(date(2017, 3, 13), holidays)
        self.assertIn(date(2017, 10, 19), holidays)  # Dia do Piauí


class BrazilParanaTest(BrazilTest):
    cal_class = BrazilParana

    def test_year_2017_state(self):
        holidays = self.cal.holidays_set(2017)
        # Aniversário do Paraná
        self.assertIn(date(2017, 12, 19), holidays)


class BrazilRioDeJaneiroTest(BrazilTest):
    cal_class = BrazilRioDeJaneiro
    test_include_consciencia_negra = True

    def test_year_2017_state(self):
        holidays = self.cal.holidays_set(2017)
        self.assertIn(date(2017, 2, 28), holidays)  # Carnaval
        self.assertIn(date(2017, 4, 23), holidays)  # Dia de São Jorge
        self.assertIn(date(2017, 10, 16), holidays)  # Dia do Comércio
        # Dia do Funcionário Público
        self.assertIn(date(2017, 10, 28), holidays)
        # Dia da Consciência Negra
        consciencia_negra = date(2017, 11, 20)
        self.assertIn(consciencia_negra, holidays)
        # check its label
        self.assertEqual(
            self.cal.get_holiday_label(consciencia_negra),
            "Dia da Consciência Negra",
        )

        # Dia de Nossa Senhora da Conceição
        self.assertIn(date(2017, 12, 8), holidays)

    def test_carnaval_label(self):
        holidays = self.cal.holidays(2017)
        holidays_dict = dict(holidays)
        label_carnaval = holidays_dict[date(2017, 2, 28)]
        self.assertEqual(label_carnaval, "Carnaval")


class BrazilRioGrandeDoNorteTest(BrazilTest):
    cal_class = BrazilRioGrandeDoNorte

    def test_year_2017_state(self):
        holidays = self.cal.holidays_set(2017)
        # Dua de São Pedro
        sao_pedro = date(2017, 6, 29)
        self.assertIn(sao_pedro, holidays)
        # Check the label
        self.assertEqual(
            self.cal.get_holiday_label(sao_pedro), "Dua de São Pedro"
        )
        # Mártires de Cunhaú e Uruaçuu
        self.assertIn(date(2017, 10, 3), holidays)


class BrazilRioGrandeDoSulTest(BrazilTest):
    cal_class = BrazilRioGrandeDoSul

    def test_year_2017_state(self):
        holidays = self.cal.holidays_set(2017)
        self.assertIn(date(2017, 9, 20), holidays)  # Revolução Farroupilha


class BrazilRondoniaTest(BrazilTest):
    cal_class = BrazilRondonia

    def test_year_2017_state(self):
        holidays = self.cal.holidays_set(2017)
        self.assertIn(date(2017, 1, 4), holidays)  # Criação do estado
        self.assertIn(date(2017, 6, 18), holidays)  # Dia do Evangélico


class BrazilRoraimaTest(BrazilTest):
    cal_class = BrazilRoraima

    def test_year_2017_state(self):
        holidays = self.cal.holidays_set(2017)
        self.assertIn(date(2017, 10, 5), holidays)  # Criação de Roraima


class BrazilSantaCatarinaTest(BrazilTest):
    cal_class = BrazilSantaCatarina

    def test_year_2017_state(self):
        holidays = self.cal.holidays_set(2017)
        # Criação da capitania, separando-se de SP
        self.assertIn(date(2017, 8, 11), holidays)
        # Dia de Santa Catarina de Alexandria
        self.assertIn(date(2017, 11, 25), holidays)


class SaoPauloStateTest(BrazilTest):
    cal_class = BrazilSaoPauloState

    def test_year_2013_state(self):
        holidays = self.cal.holidays_set(2013)
        # Revolução Constitucionalista de 1932
        self.assertIn(date(2013, 7, 9), holidays)
        # Revolução Constitucionalista de 1932
        self.assertIn(date(2013, 7, 9), holidays)


class SaoPauloCityTest(SaoPauloStateTest):
    cal_class = BrazilSaoPauloCity
    test_include_consciencia_negra = True

    def test_year_2013_city(self):
        holidays = self.cal.holidays_set(2013)
        # Aniversário da Cidade de São Paulo
        self.assertIn(date(2013, 1, 25), holidays)
        self.assertIn(date(2013, 2, 12), holidays)  # Carnaval
        # Dia da Consciência Negra
        consciencia_negra = date(2013, 11, 20)
        self.assertIn(consciencia_negra, holidays)
        # check its label
        self.assertEqual(
            self.cal.get_holiday_label(consciencia_negra),
            "Dia da Consciência Negra",
        )
        self.assertIn(date(2013, 3, 31), holidays)  # Páscoa
        self.assertIn(date(2013, 5, 30), holidays)  # Corpus Christi

        # Variable day: Good Friday
        good_friday = date(2013, 3, 29)
        self.assertIn(good_friday, holidays)  # Sexta-feira da Paixão
        # Label test
        self.assertEqual(
            self.cal.get_holiday_label(good_friday),
            "Sexta-feira da Paixão",
        )

    def test_carnaval_label(self):
        holidays = self.cal.holidays(2013)
        holidays_dict = dict(holidays)
        label_carnaval = holidays_dict[date(2013, 2, 12)]
        self.assertEqual(label_carnaval, "Carnaval")


class BrazilSergipeTest(BrazilTest):
    cal_class = BrazilSergipe

    def test_year_2017_state(self):
        holidays = self.cal.holidays_set(2017)
        # Autonomia política de Sergipe
        self.assertIn(date(2017, 7, 8), holidays)


class BrazilTocantinsTest(BrazilTest):
    cal_class = BrazilTocantins

    def test_year_2017_state(self):
        holidays = self.cal.holidays_set(2017)
        self.assertIn(date(2017, 1, 1), holidays)  # Instalação de Tocantins
        # Nossa Senhora da Natividade
        self.assertIn(date(2017, 9, 8), holidays)
        self.assertIn(date(2017, 10, 5), holidays)  # Criação de Tocantins
        # Autonomia do estado de Tocantins
        self.assertIn(date(2017, 3, 18), holidays)


class BrazilVitoriaCityTest(BrazilEspiritoSantoTest):
    """
    Vitória city is in the Espírito Santo state
    """
    cal_class = BrazilVitoriaCity

    def test_year_2017_city(self):
        holidays = self.cal.holidays_set(2017)
        # Fixed days
        self.assertIn(date(2017, 4, 24), holidays)  # Nossa Senhora da Penha
        self.assertIn(date(2017, 9, 8), holidays)  # Nossa Senhora da Vitória

        # Variable days: Corpus Christie
        corpus_christie = date(2017, 6, 15)
        self.assertIn(corpus_christie, holidays)

        # Variable days: Good friday
        good_friday = date(2017, 4, 14)
        self.assertIn(good_friday, holidays)
        # Test label
        self.assertEqual(
            self.cal.get_holiday_label(good_friday),
            "Paixão do Cristo",
        )


class BrazilVilaVelhaCityTest(BrazilEspiritoSantoTest):
    """
    Vila Velha city is in the Espírito Santo state
    """
    cal_class = BrazilVilaVelhaCity

    def test_year_2017_city(self):
        holidays = self.cal.holidays_set(2017)
        # Colonização do Solo Espírito-santense
        self.assertIn(date(2017, 5, 23), holidays)


class BrazilCariacicaCityTest(BrazilEspiritoSantoTest):
    """
    Cariacica city is in the Espírito Santo state
    """
    cal_class = BrazilCariacicaCity

    def test_year_2017_city(self):
        holidays = self.cal.holidays_set(2017)
        # Fixed days
        self.assertIn(date(2017, 4, 13), holidays)  # Nossa Senhora da Penha
        # São João Batista / Aniversãrio de Cariacica
        sao_joao = date(2017, 6, 24)
        self.assertIn(sao_joao, holidays)
        # Check São João label
        self.assertEqual(
            self.cal.get_holiday_label(sao_joao),
            "São João Batista / Aniversãrio de Cariacica"
        )

        # Variable days: Corpus Christie
        corpus_christie = date(2017, 6, 15)
        self.assertIn(corpus_christie, holidays)

        # Variable days: Good friday
        good_friday = date(2017, 4, 14)
        self.assertIn(good_friday, holidays)
        # Test label
        self.assertEqual(
            self.cal.get_holiday_label(good_friday),
            "Paixão do Cristo",
        )


class BrazilGuarapariCityTest(BrazilEspiritoSantoTest):
    """
    Guarapari city is in the Espírito Santo state
    """
    cal_class = BrazilGuarapariCity
    test_include_consciencia_negra = True

    def test_year_2017_city(self):
        holidays = self.cal.holidays_set(2017)
        self.assertIn(date(2017, 6, 29), holidays),  # São Pedro
        self.assertIn(date(2017, 9, 19), holidays),  # Emancipação de Guarapari
        self.assertIn(date(2017, 11, 29), holidays),  # Consciência Negra
        self.assertIn(date(2017, 12, 8), holidays),  # Nossa Senhora Conceição


class BrazilSerraCityTest(BrazilEspiritoSantoTest):
    """
    Serra city is in the Espírito Santo state
    """
    cal_class = BrazilSerraCity

    def test_year_2017_city(self):
        holidays = self.cal.holidays_set(2017)
        # Fixed days
        self.assertIn(date(2017, 6, 29), holidays)  # São Pedro
        self.assertIn(date(2017, 12, 8), holidays)  # Nossa Senhora Conceição
        self.assertIn(date(2017, 12, 26), holidays)  # Dia do Serrano

        # Variable days: Carnaval (Monday & Tuesday)
        self.assertIn(date(2017, 2, 27), holidays)  # Monday
        self.assertIn(date(2017, 2, 28), holidays)  # Tuesday

        # Variable days: Ash Wednesday (Quarta-feira de cinzas)
        ash_wednesday = date(2017, 3, 1)
        self.assertIn(ash_wednesday, holidays)
        # Test label
        self.assertEqual(
            self.cal.get_holiday_label(ash_wednesday),
            "Quarta-feira de cinzas",
        )

        # Variable days: Good Friday (Paixão de Cristo)
        good_friday = date(2017, 4, 14)
        self.assertIn(good_friday, holidays)
        # Test label
        self.assertEqual(
            self.cal.get_holiday_label(good_friday),
            "Paixão do Cristo",
        )

    def test_carnaval_label(self):
        holidays = self.cal.holidays(2017)
        holidays_dict = dict(holidays)
        label_carnaval = holidays_dict[date(2017, 2, 28)]
        self.assertEqual(label_carnaval, "Carnaval")


class BrazilRioBrancoCityTest(BrazilAcreTest):
    """
    Rio Branco is in the Acre state
    """
    cal_class = BrazilRioBrancoCity

    def test_year_2017_city(self):
        holidays = self.cal.holidays_set(2017)
        # Fixed days
        # Aniversário de Rio Branco
        self.assertIn(date(2017, 12, 28), holidays)


class BrazilMaceioCityTest(BrazilAlagoasTest):
    """
    Maceió is in the Alagoas state
    """
    cal_class = BrazilMaceioCity

    def test_year_2017_city(self):
        holidays = self.cal.holidays_set(2017)
        # Fixed days
        # Aniversário de Maceió
        self.assertIn(date(2017, 12, 5), holidays)


class BrazilManausCityTest(BrazilAmazonasTest):
    """
    Manaus is in the Amazonas state
    """
    cal_class = BrazilManausCity

    def test_year_2017_city(self):
        holidays = self.cal.holidays_set(2017)
        # Fixed days
        # Aniversário de Manaus
        self.assertIn(date(2017, 10, 24), holidays)


class BrazilMacapaCityTest(BrazilAmapaTest):
    """
    Macapá is in the Amapá state
    """
    cal_class = BrazilMacapaCity

    def test_year_2017_city(self):
        holidays = self.cal.holidays_set(2017)
        # Fixed days
        # Aniversário de Macapá
        self.assertIn(date(2017, 2, 4), holidays)


class BrazilSalvadorCityTest(BrazilBahiaTest):
    """
    Salvador is in the Bahia state
    """
    cal_class = BrazilSalvadorCity

    def test_year_2017_city(self):
        holidays = self.cal.holidays_set(2017)
        # Fixed days
        # Aniversário de Salvador
        self.assertIn(date(2017, 3, 29), holidays)


class BrazilFortalezaCityTest(BrazilCearaTest):
    """
    Fortaleza is in the Ceará state
    """
    cal_class = BrazilFortalezaCity

    def test_year_2017_city(self):
        holidays = self.cal.holidays_set(2017)
        # Fixed days
        # Aniversário de Fortaleza
        self.assertIn(date(2017, 4, 13), holidays)


class BrazilGoianiaCityTest(BrazilGoiasTest):
    """
    Goiânia is in the Goiás state
    """
    cal_class = BrazilGoianiaCity

    def test_year_2017_city(self):
        holidays = self.cal.holidays_set(2017)
        # Fixed days
        # Aniversário de Goiânia
        self.assertIn(date(2017, 10, 24), holidays)


class BrazilBeloHorizonteCityTest(BrazilMinasGeraisTest):
    """
    Belo Horizonte is in the Minas Gerais state
    """
    cal_class = BrazilBeloHorizonteCity

    def test_year_2017_city(self):
        holidays = self.cal.holidays_set(2017)
        # Fixed days
        # Aniversário de Belo Horizonte
        self.assertIn(date(2017, 12, 12), holidays)


class BrazilCampoGrandeCityTest(BrazilMatoGrossoDoSulTest):
    """
    Campo Grande is in the Mato Grosso do Sul state
    """
    cal_class = BrazilCampoGrandeCity

    def test_year_2017_city(self):
        holidays = self.cal.holidays_set(2017)
        # Fixed days
        # Aniversário de Campo Grande
        self.assertIn(date(2017, 8, 26), holidays)


class BrazilCuiabaCityTest(BrazilMatoGrossoTest):
    """
    Cuiabá is in the Mato Grosso state
    """
    cal_class = BrazilCuiabaCity

    def test_year_2017_city(self):
        holidays = self.cal.holidays_set(2017)
        # Fixed days
        # Aniversário de Cuiabá
        self.assertIn(date(2017, 4, 8), holidays)


class BrazilBelemCityTest(BrazilParaTest):
    """
    Belém is in the Pará state
    """
    cal_class = BrazilBelemCity

    def test_year_2017_city(self):
        holidays = self.cal.holidays_set(2017)
        # Fixed days
        # Aniversário de Belém
        self.assertIn(date(2017, 1, 12), holidays)


class BrazilJoaoPessoaCityTest(BrazilParaibaTest):
    """
    João Pessoa is in the Paraíba state
    """
    cal_class = BrazilJoaoPessoaCity

    def test_year_2017_city(self):
        holidays = self.cal.holidays_set(2017)
        # Fixed days
        # Aniversário de João Pessoa
        self.assertIn(date(2017, 8, 5), holidays)


class BrazilRecifeCityTest(BrazilPernambucoTest):
    """
    Recife is in the Pernambuco state
    """
    cal_class = BrazilRecifeCity

    def test_year_2017_city(self):
        holidays = self.cal.holidays_set(2017)
        # Fixed days
        # Aniversário de Recife
        self.assertIn(date(2017, 3, 12), holidays)


class BrazilTeresinaCityTest(BrazilPiauiTest):
    """
    Teresina is in the Piauí state
    """
    cal_class = BrazilTeresinaCity

    def test_year_2017_city(self):
        holidays = self.cal.holidays_set(2017)
        # Fixed days
        # Aniversário de Teresina
        self.assertIn(date(2017, 8, 16), holidays)


class BrazilCuritibaCityTest(BrazilParanaTest):
    """
    Curitiba is in the Paraná state
    """
    cal_class = BrazilCuritibaCity

    def test_year_2017_city(self):
        holidays = self.cal.holidays_set(2017)
        # Fixed days
        # Aniversário de Curitiba
        self.assertIn(date(2017, 3, 29), holidays)


class BrazilNatalCityTest(BrazilRioGrandeDoNorteTest):
    """
    Natal is in the Rio Grande do Norte state
    """
    cal_class = BrazilNatalCity

    def test_year_2017_city(self):
        holidays = self.cal.holidays_set(2017)
        # Fixed days
        # Aniversário de Natal
        self.assertIn(date(2017, 12, 25), holidays)


class BrazilPortoVelhoCityTest(BrazilRondoniaTest):
    """
    Porto Velho is in the Rio Grande do Norte state
    """
    cal_class = BrazilPortoVelhoCity

    def test_year_2017_city(self):
        holidays = self.cal.holidays_set(2017)
        # Fixed days
        # Aniversário de Porto Velho
        self.assertIn(date(2017, 10, 2), holidays)


class BrazilBoaVistaCityTest(BrazilRoraimaTest):
    """
    Boa Vista is in the Roraima state
    """
    cal_class = BrazilBoaVistaCity

    def test_year_2017_city(self):
        holidays = self.cal.holidays_set(2017)
        # Fixed days
        # Aniversário de Boa Vista
        self.assertIn(date(2017, 6, 9), holidays)


class BrazilPortoAlegreCityTest(BrazilRioGrandeDoSulTest):
    """
    Porto Alegre is in the Rio Grande do Sul state
    """
    cal_class = BrazilPortoAlegreCity

    def test_year_2017_city(self):
        holidays = self.cal.holidays_set(2017)
        # Fixed days
        # Aniversário de Porto Alegre
        self.assertIn(date(2017, 3, 26), holidays)


class BrazilChapecoCityTest(BrazilSantaCatarinaTest):
    """
    Chapecó is in the Santa Catarina state
    """
    cal_class = BrazilChapecoCity

    def test_year_2017_city(self):
        holidays = self.cal.holidays_set(2017)
        # Fixed days
        # Aniversário de Chapecó
        self.assertIn(date(2017, 8, 25), holidays)


class BrazilFlorianopolisCityTest(BrazilSantaCatarinaTest):
    """
    Florianópolis is in the Santa Catarina state
    """
    cal_class = BrazilFlorianopolisCity

    def test_year_2017_city(self):
        holidays = self.cal.holidays_set(2017)
        # Fixed days
        # Aniversário de Florianópolis
        self.assertIn(date(2017, 3, 23), holidays)


class BrazilJoinvilleCityTest(BrazilSantaCatarinaTest):
    """
    Joinville is in the Santa Catarina state
    """
    cal_class = BrazilJoinvilleCity

    def test_year_2017_city(self):
        holidays = self.cal.holidays_set(2017)
        # Fixed days
        # Aniversário de Joinville
        self.assertIn(date(2017, 3, 9), holidays)


class BrazilAracajuCityTest(BrazilSergipeTest):
    """
    Aracajú is in the Sergipe state
    """
    cal_class = BrazilAracajuCity

    def test_year_2017_city(self):
        holidays = self.cal.holidays_set(2017)
        # Fixed days
        # Aniversário de Aracajú
        self.assertIn(date(2017, 3, 17), holidays)


class BrazilSorocabaCityTest(SaoPauloStateTest):
    """
    Sorocaba is in the São Paulo state
    """
    cal_class = BrazilSorocabaCity

    def test_year_2017_city(self):
        holidays = self.cal.holidays_set(2017)
        # Fixed days
        # Aniversário de Sorocaba
        self.assertIn(date(2017, 8, 15), holidays)


class BrazilPalmasCityTest(BrazilTocantinsTest):
    """
    Palmas is in the Tocantins state
    """
    cal_class = BrazilPalmasCity

    def test_year_2017_city(self):
        holidays = self.cal.holidays_set(2017)
        # Fixed days
        # Aniversário de Palmas
        self.assertIn(date(2017, 5, 20), holidays)


class BrazilBankCalendarTest(BrazilTest):
    cal_class = BrazilBankCalendar

    def test_year_2017_holidays(self):
        holidays = self.cal.holidays_set(2017)
        self.assertIn(date(2017, 1, 1), holidays)  # New year
        self.assertIn(date(2017, 2, 27), holidays)  # Monday carnaval
        self.assertIn(date(2017, 2, 28), holidays)  # Tuesday carnaval
        self.assertIn(date(2017, 3, 1), holidays)  # Ash wednesday
        self.assertIn(date(2017, 4, 14), holidays)  # Good friday
        self.assertIn(date(2017, 4, 21), holidays)  # Tiradentes
        self.assertIn(date(2017, 5, 1), holidays)  # Labour day
        self.assertIn(date(2017, 6, 15), holidays)  # Corpus Christi
        self.assertIn(date(2017, 9, 7), holidays)  # Independence day
        self.assertIn(date(2017, 10, 12), holidays)  # Our Lady Aparecida
        self.assertIn(date(2017, 11, 2), holidays)  # All Souls' Day
        self.assertIn(date(2017, 11, 15), holidays)  # Republic day
        self.assertIn(date(2017, 12, 25), holidays)  # Christmas Day
        self.assertIn(date(2017, 12, 29), holidays)  # Last working day of year
        self.assertEquals(14, len(holidays))

    def test_year_2017_find_next_working_day_for_new_year(self):
        new_year = date(2017, 1, 1)
        working_day = self.cal.find_following_working_day(new_year)
        self.assertEquals(working_day, date(2017, 1, 2))

    def test_year_2017_find_next_working_day_for_monday_carnaval(self):
        monday_carnaval = date(2017, 2, 27)
        working_day = self.cal.find_following_working_day(monday_carnaval)
        self.assertEquals(working_day, date(2017, 3, 2))

    def test_year_2017_find_next_working_day_for_tuesday_carnaval(self):
        tuesday_carnaval = date(2017, 2, 28)
        working_day = self.cal.find_following_working_day(tuesday_carnaval)
        self.assertEquals(working_day, date(2017, 3, 2))

    def test_year_2017_find_next_working_day_for_good_friday(self):
        good_friday = date(2017, 4, 14)
        working_day = self.cal.find_following_working_day(good_friday)
        self.assertEquals(working_day, date(2017, 4, 17))

    def test_year_2017_find_next_working_day_for_tiradentes(self):
        tiradentes = date(2017, 4, 21)
        working_day = self.cal.find_following_working_day(tiradentes)
        self.assertEquals(working_day, date(2017, 4, 24))

    def test_year_2017_find_next_working_day_for_labour_day(self):
        labour_day = date(2017, 5, 1)
        working_day = self.cal.find_following_working_day(labour_day)
        self.assertEquals(working_day, date(2017, 5, 2))

    def test_year_2017_find_next_working_day_for_corpus_christi(self):
        corpus_christi = date(2017, 6, 15)
        working_day = self.cal.find_following_working_day(corpus_christi)
        self.assertEquals(working_day, date(2017, 6, 16))

    def test_year_2017_find_next_working_day_for_independence_day(self):
        independence_day = date(2017, 9, 7)
        working_day = self.cal.find_following_working_day(independence_day)
        self.assertEquals(working_day, date(2017, 9, 8))

    def test_year_2017_find_next_working_day_for_our_lady_aparecida(self):
        our_lady_aparecida = date(2017, 10, 12)
        working_day = self.cal.find_following_working_day(our_lady_aparecida)
        self.assertEquals(working_day, date(2017, 10, 13))

    def test_year_2017_find_next_working_day_for_our_all_souls(self):
        all_souls = date(2017, 11, 2)
        working_day = self.cal.find_following_working_day(all_souls)
        self.assertEquals(working_day, date(2017, 11, 3))

    def test_year_2017_find_next_working_day_for_our_republic_day(self):
        republic_day = date(2017, 11, 15)
        working_day = self.cal.find_following_working_day(republic_day)
        self.assertEquals(working_day, date(2017, 11, 16))

    def test_year_2017_find_next_working_day_for_our_christmas_day(self):
        christmas_day = date(2017, 12, 25)
        working_day = self.cal.find_following_working_day(christmas_day)
        self.assertEquals(working_day, date(2017, 12, 26))

    def test_year_2017_find_next_working_day_for_last_day(self):
        # last day of year for only internal bank transactions
        last_day = date(2017, 12, 29)
        working_day = self.cal.find_following_working_day(last_day)
        self.assertEquals(working_day, date(2018, 1, 2))

    def test_year_2017_find_next_working_day_for_already_working_day(self):
        already_working_day = date(2017, 7, 25)
        working_day = self.cal.find_following_working_day(already_working_day)
        self.assertEquals(working_day, date(2017, 7, 25))

    def test_carnaval_label(self):
        holidays = self.cal.holidays(2017)
        holidays_dict = dict(holidays)
        label_carnaval = holidays_dict[date(2017, 2, 28)]
        self.assertEqual(label_carnaval, "Tuesday carnaval")


class TestIBGERegister(TestCase):

    def test_register_length(self):
        # Each time another calendar will be added, this length should increase
        # This also fails when a key appears twice (typo mistake?)
        self.assertEqual(len(IBGE_REGISTER), len(IBGE_TUPLE))

    def test_no_duplicate(self):
        # Check if a class doesn't appear twice with different keys
        values = set(IBGE_REGISTER.values())
        self.assertEqual(len(IBGE_REGISTER.values()), len(values))

    def test_all_are_brazilian_classes(self):
        for key, value in IBGE_TUPLE:
            self.assertTrue(issubclass(value, Brazil))



from ..core import WesternCalendar
from ..registry_tools import iso_register


@iso_register('BE')
class Belgium(WesternCalendar):
    'Belgium'

    FIXED_HOLIDAYS = WesternCalendar.FIXED_HOLIDAYS + (
        (5, 1, "Labour Day"),
        (7, 21, "National Day"),
        (11, 11, "Armistice of 1918"),
    )

    include_easter_monday = True
    include_ascension = True
    include_whit_monday = True
    include_assumption = True
    include_all_saints = True



from datetime import date
from ..core import OrthodoxCalendar
from ..registry_tools import iso_register


@iso_register('RO')
class Romania(OrthodoxCalendar):
    'Romania'

    FIXED_HOLIDAYS = OrthodoxCalendar.FIXED_HOLIDAYS + (
        (1, 2, "Day After New Year"),
        (1, 24, "Union Day"),
        (5, 1, "Labour Day"),
        (8, 15, "Dormition of the Theotokos"),
        (11, 30, "St. Andrew's Day"),
        (12, 1, "National Day/Great Union"),
    )

    include_good_friday = True
    include_easter_sunday = True
    include_easter_monday = True
    include_whit_sunday = True
    whit_sunday_label = 'Pentecost'
    include_whit_monday = True

    include_christmas = True
    include_boxing_day = True
    boxing_day_label = 'Christmas Day'

    def get_childrens_day(self, year):
        """returns a possibly empty list of (date, holiday_name) tuples"""
        days = []
        if year >= 2017:
            actual_date = date(year, 6, 1)
            days = [(actual_date, "Children's Day")]

        return days

    def get_liberation_day(self, year):
        """returns a possibly empty list of (date, holiday_name) tuples"""
        days = []
        if year >= 1949 and year <= 1990:
            actual_date = date(year, 8, 23)
            days = [(actual_date, "Liberation from Fascist Occupation Day")]

        return days

    def get_variable_days(self, year):
        days = super().get_variable_days(year)
        days.extend(self.get_childrens_day(year))
        days.extend(self.get_liberation_day(year))
        return days



from datetime import timedelta, date
from ..core import WesternCalendar, MON, TUE, WED, THU, FRI, SAT
from ..registry_tools import iso_register


@iso_register('AR')
class Argentina(WesternCalendar):
    'Argentina'

    include_fat_tuesday = True
    fat_tuesday_label = "Carnival"
    include_good_friday = True
    include_easter_saturday = True
    include_easter_sunday = True
    include_christmas = True

    FIXED_HOLIDAYS = WesternCalendar.FIXED_HOLIDAYS + (
        (3, 24, "Día Nacional de la Memoria por la Verdad y la Justicia"),
        (5, 1, "Día del Trabajador"),
        (5, 25, "Día de la Revolución de Mayo"),
        (6, 20, "Día Paso a la Inmortalidad del General Manuel Belgrano"),
        (7, 9, "Día de la Independencia"),
        (12, 8, "Día de la Inmaculada Concepción de María"),
    )

    def get_general_guemes_day(self, year):
        """
        Día Paso a la Inmortalidad del General Martín Miguel de Güemes.

        Happens on June 17th, except:

        * if it happens on a THU, it's shifted to the next MON.
        * if it happens on a WED, it's shifted to the MON before this date.
        """
        general_guemes_day = date(year, 6, 17)

        if general_guemes_day.weekday() == THU:
            general_guemes_day = Argentina.get_first_weekday_after(
                date(year, 6, 17), MON)
        elif general_guemes_day.weekday() == WED:
            # Monday of the same week
            general_guemes_day = general_guemes_day - timedelta(days=2)
        else:
            general_guemes_day

        return (general_guemes_day,
                "Día Paso a la Inmortalidad del " +
                "General Martín Miguel de Güemes")

    def get_general_martin_day(self, year):
        """
        Día Paso a la Inmortalidad del Gral. José de San Martín

        Third MON of August.
        """
        general_martin_day = Argentina.get_nth_weekday_in_month(
            year, 8, MON, 3
        )

        return (general_martin_day,
                "Día Paso a la Inmortalidad del " +
                "Gral. José de San Martín")

    def get_soberania_day(self, year):
        """
        Día de la Soberanía Nacional

        Happens on the 3rd MON of November after the first Friday.
        """
        first_friday_november = Argentina.get_nth_weekday_in_month(
            year, 11, FRI, 1
        )

        soberania_day = Argentina.get_nth_weekday_in_month(
            year, 11, MON, n=3, start=first_friday_november
        )

        return (soberania_day, "Día de la Soberanía Nacional")

    def get_diversidad_day(self, year):
        """
        Día del Respeto a la Diversidad Cultural

        The pivot date is the 12th of October.

        * If it happens on a TUE, it's shifter on the 11th of Oct.
        * If it happens on a WED, THU, FRI or SAT, it's shifted on the first
          MON after this date.
        * Else, it's on the 12th of October.
        """
        diversidad_day = date(year, 10, 12)

        if (diversidad_day.weekday() == WED or
                diversidad_day.weekday() == THU or
                diversidad_day.weekday() == FRI or
                diversidad_day.weekday() == SAT):
            diversidad_day = Argentina.get_first_weekday_after(
                date(year, 10, 12), MON
            )
        elif diversidad_day.weekday() == TUE:
            diversidad_day = diversidad_day - timedelta(days=1)
        else:
            diversidad_day

        return (diversidad_day,
                "Día del Respeto a la Diversidad Cultural")

    def get_malvinas_day(self, year):
        """
        Día de las Malvinas

        In honour of the Veterans and the Fallen of the Malvinas war.
        https://en.wikipedia.org/wiki/Malvinas_Day

        In 2020, it was shifted to March 31st because of
        the coronavirus crisis.
        """
        label = "Día del Veterano y de los Caídos en la Guerra de Malvinas"
        if year == 2020:
            day = date(year, 3, 31)
        else:
            day = date(year, 4, 2)
        return (day, label)

    def get_variable_days(self, year):

        days = super().get_variable_days(year)
        days.append(
            (self.get_easter_sunday(year) - timedelta(days=48),
                "Carnival Lunes"))

        days.append(
            self.get_malvinas_day(year))

        days.append(
            (self.get_general_guemes_day(year)))

        days.append(
            (self.get_general_martin_day(year)))

        days.append(
            (self.get_soberania_day(year)))

        days.append(
            (self.get_diversidad_day(year)))

        return days



from .algeria import Algeria
from .benin import Benin
from .ivory_coast import IvoryCoast
from .kenya import Kenya
from .madagascar import Madagascar
from .sao_tome import SaoTomeAndPrincipe
from .south_africa import SouthAfrica
from .angola import Angola


__all__ = (
    'Algeria',
    'Benin',
    'IvoryCoast',
    'Kenya',
    'Madagascar',
    'SaoTomeAndPrincipe',
    'SouthAfrica',
    'Angola',
)



from .austria import Austria
from .belarus import Belarus
from .belgium import Belgium
from .bulgaria import Bulgaria
from .cayman_islands import CaymanIslands
from .croatia import Croatia
from .cyprus import Cyprus
from .czech_republic import CzechRepublic
from .estonia import Estonia
from .european_central_bank import EuropeanCentralBank
from .denmark import Denmark
from .finland import Finland
from .france import France, FranceAlsaceMoselle
from .greece import Greece
from .hungary import Hungary
from .iceland import Iceland
from .ireland import Ireland
from .italy import Italy
from .latvia import Latvia
from .lithuania import Lithuania
from .luxembourg import Luxembourg
from .malta import Malta
from .netherlands import Netherlands
from .norway import Norway
from .poland import Poland
from .portugal import Portugal
from .romania import Romania
from .russia import Russia
from .serbia import Serbia
from .slovakia import Slovakia
from .slovenia import Slovenia
from .spain import Spain, Catalonia
from .sweden import Sweden
from .ukraine import Ukraine
from .united_kingdom import UnitedKingdom, UnitedKingdomNorthernIreland
from .turkey import Turkey

# Germany
from .germany import (
    Germany, BadenWurttemberg, Bavaria, Berlin, Brandenburg, Bremen,
    Hamburg, Hesse, MecklenburgVorpommern, LowerSaxony,
    NorthRhineWestphalia, RhinelandPalatinate, Saarland, Saxony,
    SaxonyAnhalt, SchleswigHolstein, Thuringia
)

# Scotland
from .scotland import (
    Scotland, Aberdeen, Angus, Arbroath, Ayr, CarnoustieMonifieth, Clydebank,
    DumfriesGalloway, Dundee, EastDunbartonshire, Edinburgh, Elgin, Falkirk,
    Fife, Galashiels, Glasgow, Hawick, Inverclyde, Inverness, Kilmarnock,
    Lanark, Linlithgow, Lochaber, NorthLanarkshire, Paisley, Perth,
    ScottishBorders, SouthLanarkshire, Stirling, WestDunbartonshire
)

# Switzerland
from .switzerland import (
    Switzerland, Aargau, AppenzellInnerrhoden, AppenzellAusserrhoden, Bern,
    BaselLandschaft, BaselStadt, Fribourg, Geneva, Glarus, Graubunden, Jura,
    Luzern, Neuchatel, Nidwalden, Obwalden, StGallen, Schaffhausen, Solothurn,
    Schwyz, Thurgau, Ticino, Uri, Vaud, Valais, Zug, Zurich
)

__all__ = (
    'Austria',
    'Belarus',
    'Belgium',
    'Bulgaria',
    'Catalonia',
    'CaymanIslands',
    'Croatia',
    'Cyprus',
    'CzechRepublic',
    'Denmark',
    'Estonia',
    'EuropeanCentralBank',
    'Finland',
    'France',
    'FranceAlsaceMoselle',
    'Greece',
    'Hungary',
    'Iceland',
    'Ireland',
    'Italy',
    'Latvia',
    'Lithuania',
    'Luxembourg',
    'Malta',
    'Netherlands',
    'Norway',
    'Poland',
    'Portugal',
    'Romania',
    'Russia',
    'Serbia',
    'Slovakia',
    'Slovenia',
    'Spain',
    'Sweden',
    'Switzerland',
    'Ukraine',
    'UnitedKingdom',
    'UnitedKingdomNorthernIreland',
    'Turkey',

    # Germany
    'Germany', 'BadenWurttemberg', 'Bavaria', 'Berlin', 'Brandenburg',
    'Bremen', 'Hamburg', 'Hesse', 'MecklenburgVorpommern', 'LowerSaxony',
    'NorthRhineWestphalia', 'RhinelandPalatinate', 'Saarland', 'Saxony',
    'SaxonyAnhalt', 'SchleswigHolstein', 'Thuringia',
    # Scotland
    'Scotland',
    'Aberdeen', 'Angus', 'Arbroath', 'Ayr', 'CarnoustieMonifieth',
    'Clydebank', 'DumfriesGalloway', 'Dundee', 'EastDunbartonshire',
    'Edinburgh', 'Elgin', 'Falkirk', 'Fife', 'Galashiels', 'Glasgow',
    'Hawick', 'Inverclyde', 'Inverness', 'Kilmarnock', 'Lanark', 'Linlithgow',
    'Lochaber', 'NorthLanarkshire', 'Paisley', 'Perth', 'ScottishBorders',
    'SouthLanarkshire', 'Stirling', 'WestDunbartonshire',
    # Switzerland
    'Switzerland',
    'Aargau', 'AppenzellInnerrhoden', 'AppenzellAusserrhoden', 'Bern',
    'BaselLandschaft', 'BaselStadt', 'Fribourg', 'Geneva', 'Glarus',
    'Graubunden', 'Jura', 'Luzern', 'Neuchatel', 'Nidwalden', 'Obwalden',
    'StGallen', 'Schaffhausen', 'Solothurn', 'Schwyz', 'Thurgau', 'Ticino',
    'Uri', 'Vaud', 'Valais', 'Zug', 'Zurich',
)



from unittest import TestCase
from ..europe import (
    Austria, Belgium, Bulgaria, Croatia, Cyprus, CzechRepublic, Estonia,
    Denmark, Finland, France,
    # FranceAlsaceMoselle,  # TODO: Should we add it to the registry?
    Greece, Hungary, Iceland, Ireland, Italy, Latvia, Lithuania, Luxembourg,
    Malta, Netherlands, Norway, Poland, Portugal, Romania, Russia, Slovakia,
    Slovenia, Spain,
    # Catalonia,  # TODO: Add it to registry
    Sweden, UnitedKingdom,
    UnitedKingdomNorthernIreland,
)

# Switzerland
from ..europe import (
    Switzerland,
    Aargau, AppenzellInnerrhoden, AppenzellAusserrhoden, Bern, BaselLandschaft,
    BaselStadt, Fribourg, Geneva, Glarus, Graubunden, Jura, Luzern, Neuchatel,
    Nidwalden, Obwalden, StGallen, Schaffhausen, Solothurn, Schwyz, Thurgau,
    Ticino, Uri, Vaud, Valais, Zug, Zurich
)

# Germany
from ..europe import (
    Germany, BadenWurttemberg, Bavaria, Berlin, Brandenburg, Bremen,
    Hamburg, Hesse, MecklenburgVorpommern, LowerSaxony,
    NorthRhineWestphalia, RhinelandPalatinate, Saarland, Saxony,
    SaxonyAnhalt, SchleswigHolstein, Thuringia
)

from ..registry import registry

classes = (v for k, v in registry.region_registry.items())
classes = list(classes)

GERMANY_REGION_CLASSES = (
    BadenWurttemberg, Bavaria, Berlin, Brandenburg, Bremen,
    Hamburg, Hesse, MecklenburgVorpommern, LowerSaxony,
    NorthRhineWestphalia, RhinelandPalatinate, Saarland, Saxony,
    SaxonyAnhalt, SchleswigHolstein, Thuringia
)

SWITZERLAND_REGION_CLASSES = (
    Aargau, AppenzellInnerrhoden, AppenzellAusserrhoden, Bern, BaselLandschaft,
    BaselStadt, Fribourg, Geneva, Glarus, Graubunden, Jura, Luzern, Neuchatel,
    Nidwalden, Obwalden, StGallen, Schaffhausen, Solothurn, Schwyz, Thurgau,
    Ticino, Uri, Vaud, Valais, Zug, Zurich
)


class RegistryEurope(TestCase):
    def test_europe(self):
        self.assertIn(Austria, classes)
        self.assertIn(Belgium, classes)
        self.assertIn(Bulgaria, classes)
        self.assertIn(Croatia, classes)
        self.assertIn(Cyprus, classes)
        self.assertIn(CzechRepublic, classes)
        self.assertIn(Estonia, classes)
        self.assertIn(Denmark, classes)
        self.assertIn(Finland, classes)
        self.assertIn(France,  classes)
        self.assertIn(Greece, classes)
        self.assertIn(Hungary, classes)
        self.assertIn(Iceland, classes)
        self.assertIn(Ireland, classes)
        self.assertIn(Italy, classes)
        self.assertIn(Latvia, classes)
        self.assertIn(Lithuania, classes)
        self.assertIn(Luxembourg, classes)
        self.assertIn(Malta, classes)
        self.assertIn(Netherlands, classes)
        self.assertIn(Norway, classes)
        self.assertIn(Poland, classes)
        self.assertIn(Portugal, classes)
        self.assertIn(Romania, classes)
        self.assertIn(Russia, classes)
        self.assertIn(Slovakia, classes)
        self.assertIn(Slovenia, classes)
        self.assertIn(Spain,  classes)
        self.assertIn(Sweden, classes)
        self.assertIn(Switzerland, classes)
        self.assertIn(Vaud, classes)
        self.assertIn(Geneva, classes)
        self.assertIn(UnitedKingdom, classes)
        self.assertIn(UnitedKingdomNorthernIreland, classes)
        # Germany & Länders
        self.assertIn(Germany, classes)
        for klass in GERMANY_REGION_CLASSES:
            self.assertIn(klass, classes)
        for klass in SWITZERLAND_REGION_CLASSES:
            self.assertIn(klass, classes)

    def test_germany_subregion(self):
        # Get all the subregions
        classes = (v for k, v in registry.get_subregions('DE').items())
        classes = list(classes)
        for klass in GERMANY_REGION_CLASSES:
            self.assertIn(klass, classes)

    def test_switzerland_subregion(self):
        # Get all the subregions
        classes = (v for k, v in registry.get_subregions('CH').items())
        classes = list(classes)
        for klass in SWITZERLAND_REGION_CLASSES:
            self.assertIn(klass, classes)

    def test_slovenia_code(self):
        # Source: https://github.com/peopledoc/workalendar/pull/291
        self.assertEqual(registry.region_registry['SI'], Slovenia)



from ..registry_tools import iso_register
from .core import UnitedStates


@iso_register('US-NH')
class NewHampshire(UnitedStates):
    """New Hampshire"""
    include_thanksgiving_friday = True
    martin_luther_king_label = "Martin Luther King, Jr. Civil Rights Day"



from unittest import TestCase

from ..core import Calendar
from ..exceptions import ISORegistryError
from ..registry import IsoRegistry


class RegionCalendar(Calendar):
    'Region'


class SubRegionCalendar(Calendar):
    'Sub Region'


class NotACalendarClass:
    "Not a Calendar"


class NonStandardRegistryTest(TestCase):

    def setUp(self):
        self.region = RegionCalendar
        self.subregion = SubRegionCalendar

    def test_region_registry(self):
        registry = IsoRegistry(load_standard_modules=False)
        self.assertEqual(0, len(registry.region_registry.items()))
        registry.register('RE', self.region)
        self.assertEqual(1, len(registry.region_registry.items()))
        self.assertEqual(RegionCalendar, registry.region_registry['RE'])

    def test_register_non_calendar(self):
        registry = IsoRegistry(load_standard_modules=False)
        with self.assertRaises(ISORegistryError):
            registry.register("NAC", NotACalendarClass)

    def test_get(self):
        registry = IsoRegistry(load_standard_modules=False)
        registry.register('RE', self.region)
        registry.register('RE-SR', self.subregion)
        calendar_class = registry.get('RE')
        self.assertEqual(calendar_class, RegionCalendar)
        # Subregion
        calendar_class = registry.get('RE-SR')
        self.assertEqual(calendar_class, SubRegionCalendar)
        # Unknown code/region
        self.assertIsNone(registry.get('XX'))

    def test_get_subregions(self):
        registry = IsoRegistry(load_standard_modules=False)
        registry.register('RE', self.region)
        registry.register('RE-SR', self.subregion)
        registry.register('OR-SR', self.subregion)
        subregions = registry.get_subregions('RE')
        # Only one sub-region here
        self.assertEqual(1, len(subregions))
        self.assertIn('RE-SR', subregions)

    def test_get_calendars(self):
        registry = IsoRegistry(load_standard_modules=False)
        registry.register('RE', self.region)
        registry.register('RE-SR', self.subregion)
        registry.register('OR-SR', self.subregion)
        calendars = registry.get_calendars(['RE'], include_subregions=True)
        self.assertEqual(2, len(calendars))
        self.assertIn('RE', calendars)
        self.assertIn('RE-SR', calendars)
        calendars = registry.get_calendars(['RE'], include_subregions=False)
        self.assertEqual(1, len(calendars))
        self.assertIn('RE', calendars)

    def test_get_calendars_unknown(self):
        registry = IsoRegistry(load_standard_modules=False)
        registry.register('RE', self.region)
        calendars = registry.get_calendars(['XX'])
        self.assertEqual(calendars, {})

    def test_get_calendars_with_subregions(self):
        registry = IsoRegistry(load_standard_modules=False)
        registry.register('RE', self.region)
        registry.register('RE2', self.region)
        registry.register('RE-SR', self.subregion)
        calendars = registry.get_calendars(
            ['RE2', "RE-SR"], include_subregions=True)
        self.assertEqual(2, len(calendars))
        self.assertIn('RE2', calendars)
        self.assertIn('RE-SR', calendars)
        calendars = registry.get_calendars(
            ['RE2', "RE-SR"], include_subregions=False)
        self.assertEqual(2, len(calendars))
        self.assertIn('RE2', calendars)
        self.assertIn('RE-SR', calendars)

        # Only a subregion
        calendars = registry.get_calendars(["RE-SR"], include_subregions=True)
        self.assertEqual(1, len(calendars))
        self.assertIn('RE-SR', calendars)

    def test_get_calendars_empty_arg(self):
        registry = IsoRegistry(load_standard_modules=False)
        # 3 regions, one sub-region
        registry.register('RE', self.region)
        registry.register('RE2', self.region)
        registry.register('RE3', self.region)
        registry.register('RE-SR', self.subregion)
        # Empty arg, no subregions
        calendars = registry.get_calendars([], include_subregions=False)
        self.assertEqual(len(calendars), 3)
        self.assertEqual(set({"RE", "RE2", "RE3"}), set(calendars.keys()))
        # Empty arg, with subregions
        calendars = registry.get_calendars([], include_subregions=True)
        self.assertEqual(len(calendars), 4)
        self.assertEqual(
            set({"RE", "RE2", "RE3", "RE-SR"}),
            set(calendars.keys())
        )

    def test_get_calendars_no_arg(self):
        registry = IsoRegistry(load_standard_modules=False)
        # 3 regions, one sub-region
        registry.register('RE', self.region)
        registry.register('RE2', self.region)
        registry.register('RE3', self.region)
        registry.register('RE-SR', self.subregion)

        # Should be equivalent to [] + no subregions
        calendars = registry.get_calendars()
        self.assertEqual(len(calendars), 3)
        self.assertEqual(set({"RE", "RE2", "RE3"}), set(calendars.keys()))

        # Should be equivalent to [] + include subregions
        calendars = registry.get_calendars(include_subregions=True)
        self.assertEqual(len(calendars), 4)
        self.assertEqual(
            set({"RE", "RE2", "RE3", "RE-SR"}),
            set(calendars.keys()))



from ..core import IslamicCalendar
from ..registry_tools import iso_register


@iso_register('QA')
class Qatar(IslamicCalendar):
    "Qatar"

    include_new_years_day = False

    FIXED_HOLIDAYS = (
        (12, 18, "National Day"),
    )
    include_start_ramadan = True
    include_eid_al_fitr = True
    length_eid_al_fitr = 4
    include_eid_al_adha = True
    length_eid_al_adha = 4



from .australia import (
    Australia,
    AustralianCapitalTerritory,
    NewSouthWales,
    NorthernTerritory,
    Queensland,
    SouthAustralia,
    Tasmania,
    Hobart,
    Victoria,
    WesternAustralia
)
from .marshall_islands import MarshallIslands
from .new_zealand import NewZealand


__all__ = (
    # Australia and al.
    'Australia',
    'AustralianCapitalTerritory',
    'NewSouthWales',
    'NorthernTerritory',
    'Queensland',
    'SouthAustralia',
    'Tasmania',
    'Hobart',
    'Victoria',
    'WesternAustralia',
    # Other oceanian countries
    'MarshallIslands',
    'NewZealand',
)



from datetime import date

from ..core import WesternCalendar, SUN, MON, SAT
from ..registry_tools import iso_register


@iso_register('CA')
class Canada(WesternCalendar):
    "Canada"
    FIXED_HOLIDAYS = WesternCalendar.FIXED_HOLIDAYS + (
        (7, 1, "Canada Day"),
    )
    shift_new_years_day = True

    def get_variable_days(self, year):
        # usual variable days
        days = super().get_variable_days(year)
        days.append(
            (Canada.get_nth_weekday_in_month(year, 9, MON, 1), "Labor Day")
        )
        # Canada day
        canadaday = date(year, 7, 1)
        if canadaday.weekday() in self.get_weekend_days():
            shift = self.find_following_working_day(canadaday)
            days.append((shift, "Canada Day Shift"))
        christmas = date(year, 12, 25)
        if christmas.weekday() in self.get_weekend_days():
            shift = self.find_following_working_day(christmas)
            days.append((shift, "Christmas Shift"))
        return days


class LateFamilyDayMixin:
    "3rd Monday of February"

    def get_family_day(self, year, label="Family Day"):
        return (self.get_nth_weekday_in_month(year, 2, MON, 3), label)


class VictoriaDayMixin:
    "Monday preceding the 25th of May"

    def get_victoria_day(self, year):
        for day in range(18, 25):
            if date(year, 5, day).weekday() == MON:
                return (date(year, 5, day), "Victoria Day")


class AugustCivicHolidayMixin:
    "1st Monday of August; different names depending on location"

    def get_civic_holiday(self, year, label="Civic Holiday"):
        return (self.get_nth_weekday_in_month(year, 8, MON), label)


class ThanksgivingMixin:
    "2nd Monday of October"

    def get_thanksgiving(self, year):
        thanksgiving = self.get_nth_weekday_in_month(year, 10, MON, 2)
        return (thanksgiving, "Thanksgiving")


class BoxingDayMixin:
    "26th of December; shift to next working day"

    def get_boxing_day(self, year):
        boxingday = date(year, 12, 26)
        if boxingday.weekday() == MON:
            days = [(boxingday, "Boxing Day"), (date(year, 12, 27),
                    "Boxing Day (Shift)")]
        elif boxingday.weekday() == SAT or boxingday.weekday() == SUN:
            days = [(boxingday, "Boxing Day"), (date(year, 12, 28),
                    "Boxing Day (Shift)")]
        else:
            days = [(boxingday, "Boxing Day")]
        return days


class StJeanBaptisteMixin:
    "24th of June; shift to next working day"

    def get_st_jean(self, year):
        stjean = date(year, 6, 24)
        if stjean.weekday() in self.get_weekend_days():
            days = [(stjean, "St Jean Baptiste"),
                    (self.find_following_working_day(stjean),
                     "St Jean Baptiste (Shift)")]
        else:
            days = [(stjean, "St Jean Baptiste")]
        return days


class RemembranceDayShiftMixin:
    "11th of November; shift to next day"
    def get_remembrance_day(self, year):
        remembranceday = date(year, 11, 11)
        if remembranceday.weekday() in self.get_weekend_days():
            days = [(remembranceday, "Remembrance Day"),
                    (self.find_following_working_day(remembranceday),
                    "Remembrance Day (Shift)")]
        else:
            days = [(remembranceday, "Remembrance Day")]
        return days


@iso_register('CA-ON')
class Ontario(BoxingDayMixin, ThanksgivingMixin, VictoriaDayMixin,
              LateFamilyDayMixin, AugustCivicHolidayMixin,
              Canada):
    "Ontario"
    include_good_friday = True

    def get_variable_days(self, year):
        days = super().get_variable_days(year)
        days.extend([
            (self.get_family_day(year)),
            (self.get_victoria_day(year)),
            (self.get_civic_holiday(year, "Civic Holiday (Not for all)")),
            (self.get_thanksgiving(year)),
        ])
        days.extend(self.get_boxing_day(year))

        return days


@iso_register('CA-QC')
class Quebec(VictoriaDayMixin, StJeanBaptisteMixin, ThanksgivingMixin, Canada):
    "Quebec"
    include_easter_monday = True

    def get_variable_days(self, year):
        days = super().get_variable_days(year)
        days.extend([
            (self.get_victoria_day(year)),
            (self.get_thanksgiving(year)),
        ])
        days.extend(self.get_st_jean(year))
        return days


@iso_register('CA-BC')
class BritishColumbia(VictoriaDayMixin, AugustCivicHolidayMixin,
                      ThanksgivingMixin, Canada):
    "British Columbia"

    include_good_friday = True

    FIXED_HOLIDAYS = Canada.FIXED_HOLIDAYS + (
        (11, 11, "Remembrance Day"),
    )

    def get_family_day(self, year):
        """
        Return Family Day for British Columbia.

        From 2013 to 2018, Family Day was on 2nd MON of February
        As of 2019, Family Day happens on 3rd MON of February
        """
        label = "Family Day"
        if year >= 2019:
            return (self.get_nth_weekday_in_month(year, 2, MON, 3), label)
        return (self.get_nth_weekday_in_month(year, 2, MON, 2), label)

    def get_variable_days(self, year):
        days = super().get_variable_days(year)
        if year >= 2013:
            days.append(self.get_family_day(year))

        days.extend([
            (self.get_victoria_day(year)),
            (self.get_civic_holiday(year, "British Columbia Day")),
            (self.get_thanksgiving(year)),
        ])
        return days


@iso_register('CA-AB')
class Alberta(LateFamilyDayMixin, VictoriaDayMixin, ThanksgivingMixin, Canada):
    "Alberta"
    include_good_friday = True

    FIXED_HOLIDAYS = Canada.FIXED_HOLIDAYS + (
        (11, 11, "Remembrance Day"),
    )

    def get_variable_days(self, year):
        days = super().get_variable_days(year)
        days.extend([
            (self.get_family_day(year)),
            (self.get_victoria_day(year)),
            (self.get_thanksgiving(year)),
        ])
        return days


@iso_register('CA-SK')
class Saskatchewan(LateFamilyDayMixin, VictoriaDayMixin,
                   RemembranceDayShiftMixin, AugustCivicHolidayMixin,
                   ThanksgivingMixin, Canada):
    "Saskatchewan"
    include_good_friday = True

    def get_variable_days(self, year):
        days = super().get_variable_days(year)
        days.extend([
            (self.get_family_day(year)),
            (self.get_victoria_day(year)),
            (self.get_civic_holiday(year)),
            (self.get_thanksgiving(year)),
        ])
        days.extend(self.get_remembrance_day(year))
        return days


@iso_register('CA-MB')
class Manitoba(LateFamilyDayMixin, VictoriaDayMixin,
               AugustCivicHolidayMixin, ThanksgivingMixin,
               Canada):
    "Manitoba"
    include_good_friday = True

    def get_variable_days(self, year):
        days = super().get_variable_days(year)
        days.extend([
            (self.get_family_day(year, "Louis Riel Day")),
            (self.get_victoria_day(year)),
            (self.get_civic_holiday(year)),
            (self.get_thanksgiving(year)),
        ])
        return days


@iso_register('CA-NB')
class NewBrunswick(AugustCivicHolidayMixin, Canada):
    "New Brunswick"

    FIXED_HOLIDAYS = Canada.FIXED_HOLIDAYS + (
        (11, 11, "Remembrance Day"),
    )

    include_good_friday = True

    def get_variable_days(self, year):
        days = super().get_variable_days(year)
        days.append(self.get_civic_holiday(year))
        return days


@iso_register('CA-NS')
class NovaScotia(RemembranceDayShiftMixin, LateFamilyDayMixin, Canada):
    "Nova Scotia"

    include_good_friday = True

    def get_variable_days(self, year):
        days = super().get_variable_days(year)
        days.extend(self.get_remembrance_day(year))
        if year >= 2015:
            days.append(self.get_family_day(year, "Viola Desmond Day"))
        return days


@iso_register('CA-PE')
class PrinceEdwardIsland(LateFamilyDayMixin, RemembranceDayShiftMixin, Canada):
    "Prince Edward Island"

    include_good_friday = True

    def get_variable_days(self, year):
        days = super().get_variable_days(year)
        days.append((self.get_family_day(year, "Islander Day")))
        days.extend(self.get_remembrance_day(year))
        return days


@iso_register('CA-NL')
class Newfoundland(Canada):
    "Newfoundland and Labrador"
    include_good_friday = True


@iso_register('CA-YT')
class Yukon(VictoriaDayMixin, ThanksgivingMixin, Canada):
    "Yukon"

    FIXED_HOLIDAYS = Canada.FIXED_HOLIDAYS + (
        (11, 11, "Remembrance Day"),
    )

    include_good_friday = True

    def get_variable_days(self, year):
        days = super().get_variable_days(year)
        days.extend([
            (self.get_nth_weekday_in_month(year, 8, MON, 3), "Discovery Day"),
            (self.get_victoria_day(year)),
            (self.get_thanksgiving(year)),
        ])
        return days


@iso_register('CA-NT')
class NorthwestTerritories(RemembranceDayShiftMixin, VictoriaDayMixin,
                           ThanksgivingMixin, Canada):
    "Northwest Territories"

    FIXED_HOLIDAYS = Canada.FIXED_HOLIDAYS + (
        (6, 21, "National Aboriginal Day"),
    )

    include_good_friday = True

    def get_variable_days(self, year):
        days = super().get_variable_days(year)
        days.extend([
            (self.get_victoria_day(year)),
            (self.get_thanksgiving(year)),
        ])
        days.extend(self.get_remembrance_day(year))
        return days


@iso_register('CA-NU')
class Nunavut(VictoriaDayMixin, ThanksgivingMixin, RemembranceDayShiftMixin,
              Canada):
    "Nunavut"
    include_good_friday = True

    def get_variable_days(self, year):
        days = super().get_variable_days(year)
        days.extend([
            (self.get_victoria_day(year)),
            (self.get_thanksgiving(year)),
        ])

        days.extend(self.get_remembrance_day(year))

        nuvanutday = date(year, 7, 9)
        days.append((nuvanutday, "Nuvanut Day"))
        if nuvanutday.weekday() == SUN:
            days.append((date(year, 7, 10), "Nuvanut Day (Shift)"))
        return days



from datetime import date

from . import GenericCalendarTest
from ..europe import Turkey


class TurkeyTest(GenericCalendarTest):
    cal_class = Turkey

    def test_year_new_year_shift(self):
        holidays = self.cal.holidays_set(2012)
        self.assertIn(date(2012, 1, 1), holidays)
        self.assertIn(date(2012, 1, 2), holidays)
        holidays = self.cal.holidays_set(2013)
        self.assertIn(date(2013, 1, 1), holidays)
        self.assertNotIn(date(2013, 1, 2), holidays)

    def test_year_2014(self):
        holidays = self.cal.holidays_set(2014)
        # Fixed days section:
        # 1. New years Day
        self.assertIn(date(2014, 1, 1), holidays)
        # 2. National Sovereignty and Children's Day
        self.assertIn(date(2014, 4, 23), holidays)
        # 3. Labor and Solidarity Day
        self.assertIn(date(2014, 5, 1), holidays)
        # 4. Commemoration of Atatürk, Youth and Sports Day
        self.assertIn(date(2014, 5, 19), holidays)
        # 5. Democracy and National Unity Day
        self.assertIn(date(2014, 7, 15), holidays)
        # 6. Victory Day
        self.assertIn(date(2014, 8, 30), holidays)
        # 7. Republic Day
        self.assertIn(date(2014, 10, 29), holidays)

    def test_year_2015(self):
        holidays = self.cal.holidays_set(2015)
        # Fixed days section:
        # 1. New years Day
        self.assertIn(date(2015, 1, 1), holidays)
        # 2. National Sovereignty and Children's Day
        self.assertIn(date(2015, 4, 23), holidays)
        # 3. Labor and Solidarity Day
        self.assertIn(date(2015, 5, 1), holidays)
        # 4. Commemoration of Atatürk, Youth and Sports Day
        self.assertIn(date(2015, 5, 19), holidays)
        # 5. Democracy and National Unity Day
        self.assertIn(date(2015, 7, 15), holidays)
        # 6. Victory Day
        self.assertIn(date(2015, 8, 30), holidays)
        # 7. Republic Day
        self.assertIn(date(2015, 10, 29), holidays)

    def test_year_2019(self):
        holidays = self.cal.holidays_set(2019)
        # Fixed days section:
        # 1. New years Day
        self.assertIn(date(2019, 1, 1), holidays)
        # 2. National Sovereignty and Children's Day
        self.assertIn(date(2019, 4, 23), holidays)
        # 3. Labor and Solidarity Day
        self.assertIn(date(2019, 5, 1), holidays)
        # 4. Commemoration of Atatürk, Youth and Sports Day
        self.assertIn(date(2019, 5, 19), holidays)
        # 5. Democracy and National Unity Day
        self.assertIn(date(2019, 7, 15), holidays)
        # 6. Victory Day
        self.assertIn(date(2019, 8, 30), holidays)
        # 7. Republic Day
        self.assertIn(date(2019, 10, 29), holidays)

        # Religious days
        # Ramadan Feast - 3 days
        self.assertIn(date(2019, 6, 4), holidays)
        self.assertIn(date(2019, 6, 5), holidays)
        self.assertIn(date(2019, 6, 6), holidays)
        # Ramadan Feast - 4 days
        self.assertIn(date(2019, 8, 11), holidays)
        self.assertIn(date(2019, 8, 12), holidays)
        self.assertIn(date(2019, 8, 13), holidays)
        self.assertIn(date(2019, 8, 14), holidays)



from datetime import timedelta
from ..core import OrthodoxCalendar
from ..registry_tools import iso_register


@iso_register('BY')
class Belarus(OrthodoxCalendar):
    'Belarus'
    "as of http://president.gov.by/en/holidays_en/"

    include_christmas = False

    FIXED_HOLIDAYS = OrthodoxCalendar.FIXED_HOLIDAYS + (
        (1, 2, "Day After New Year"),
        (1, 7, "Christmas (Orthodox)"),
        (3, 8, "International Women's Day"),
        (5, 1, "Labour Day"),
        (5, 9, "Victory Day"),
        (7, 3, "Republic Day"),
        (11, 7, "October Revolution Day"),
        (12, 25, "Christmas (Catholic)"),
    )
    # Radonitsa
    # https://en.wikipedia.org/wiki/Radonitsa

    def get_radonitsa(self, year):
        # 9 Days after the Orthodox easter sunday
        return self.get_easter_sunday(year) + timedelta(days=9)

    def get_variable_days(self, year):
        days = super().get_variable_days(year)
        days.append((self.get_radonitsa(year), "Radonitsa"))
        return days



from datetime import timedelta
from ..core import WesternCalendar
from ..registry_tools import iso_register


@iso_register('CY')
class Cyprus(WesternCalendar):
    'Cyprus'

    include_epiphany = True
    include_clean_monday = True
    include_good_friday = True
    include_easter_saturday = True
    include_easter_sunday = True
    include_easter_monday = True
    include_whit_monday = True
    whit_monday_label = 'Pentecost Monday'
    include_christmas_eve = True
    include_christmas_day = True
    include_boxing_day = True

    FIXED_HOLIDAYS = WesternCalendar.FIXED_HOLIDAYS + (
        (3, 25, "Greek Independence Day"),
        (4, 1, "Cyprus National Day"),
        (5, 1, "Labour Day"),
        (7, 15, "Dormition of the Theotokos"),
        (10, 1, "Cyprus Independence Day"),
        (10, 28, "Greek National Day"),
    )

    def get_variable_days(self, year):
        days = super().get_variable_days(year)
        days.append((self.get_easter_monday(year) +
                     timedelta(days=1), "Easter Tuesday"))
        return days



"""
Scotland calendar mixins.

There are so many of them that it became necessary to move them to a different
module.
"""
from ....core import MON, FRI
from datetime import timedelta

from .spring_holiday import (
    SpringHolidayFirstMondayApril,
    SpringHolidaySecondMondayApril,
    SpringHolidayTuesdayAfterFirstMondayMay,
    SpringHolidayLastMondayMay,
    SpringHolidayFirstMondayJune,
)

from .fair_holiday import (
    FairHolidayLastMondayJune,
    FairHolidayFirstMondayJuly,
    FairHolidaySecondMondayJuly,
    FairHolidayThirdMondayJuly,
    FairHolidayLastMondayJuly,
    FairHolidayFourthFridayJuly,
    FairHolidayFirstMondayAugust,
)

from .victoria_day import (
    VictoriaDayFourthMondayMay,
    VictoriaDayLastMondayMay,
    VictoriaDayFirstMondayJune,
)

from .autumn_holiday import (
    AutumnHolidayLastMondaySeptember,
    AutumnHolidayFirstMondayOctober,
    AutumnHolidaySecondMondayOctober,
    AutumnHolidayThirdMondayOctober,
)


class LateSummer:
    def get_variable_days(self, year):
        """
        Add Late Summer holiday (First Monday of September)
        """
        days = super().get_variable_days(year)
        days.append((
            self.get_nth_weekday_in_month(year, 9, MON),
            "Late Summer Holiday"
        ))
        return days


class BattleStirlingBridge:
    def get_variable_days(self, year):
        """
        Add Battle of Stirling Bridge holiday (Second Monday of September)
        """
        days = super().get_variable_days(year)
        days.append((
            self.get_nth_weekday_in_month(year, 9, MON, 2),
            "Battle of Stirling Bridge Holiday"
        ))
        return days


class AyrGoldCup:
    def get_variable_days(self, year):
        days = super().get_variable_days(year)
        # Ayr Gold Cup
        gold_cup_friday = self.get_nth_weekday_in_month(year, 9, FRI, 3)
        days.append(
            (gold_cup_friday, "Ayr Gold Cup Friday")
        )
        days.append(
            (gold_cup_friday + timedelta(days=3), "Ayr Gold Cup Monday")
        )
        return days


__all__ = [
    'AyrGoldCup',
    'SpringHolidayFirstMondayApril',
    'SpringHolidaySecondMondayApril',
    'SpringHolidayTuesdayAfterFirstMondayMay',
    'SpringHolidayLastMondayMay',
    'SpringHolidayFirstMondayJune',
    'VictoriaDayFourthMondayMay',
    'VictoriaDayLastMondayMay',
    'VictoriaDayTuesdayAfterFirstMondayMay',
    'VictoriaDayFirstMondayJune',
    'FairHolidayLastMondayJune',
    'FairHolidayFirstMondayJuly',
    'FairHolidaySecondMondayJuly',
    'FairHolidayThirdMondayJuly',
    'FairHolidayLastMondayJuly',
    'FairHolidayFourthFridayJuly',
    'FairHolidayFirstMondayAugust',
    'AutumnHolidayLastMondaySeptember',
    'AutumnHolidayFirstMondayOctober',
    'AutumnHolidaySecondMondayOctober',
    'AutumnHolidayThirdMondayOctober',
]



from datetime import timedelta
from ..core import IslamicCalendar, SAT, SUN
from ..registry_tools import iso_register


@iso_register('TR')
class Turkey(IslamicCalendar):
    'Turkey'
    shift_new_years_day = True
    # Even though they're using an islamic calendar, the work week is MON->FRI
    WEEKEND_DAYS = (SAT, SUN)

    # Islamic Holidays
    include_eid_al_fitr = True
    length_eid_al_fitr = 3
    include_eid_al_adha = True
    length_eid_al_adha = 4

    FIXED_HOLIDAYS = IslamicCalendar.FIXED_HOLIDAYS + (
        (4, 23, "National Sovereignty and Children's Day"),
        (5, 1, "Labor and Solidarity Day"),
        (5, 19, "Commemoration of Atatürk, Youth and Sports Day"),
        (7, 15, "Democracy and National Unity Day"),
        (8, 30, "Victory Day"),
        (10, 29, "Republic Day"),
    )

    def get_delta_islamic_holidays(self, year):
        """
        Turkey Islamic holidays are shifted by one day every year.
        """
        return timedelta(days=-1)



"""
West Virginia

Christmas Eve and New Years Eve are considered as half-holidays. By default,
they're not included as "non-working days". If for your personal use you want
to include them, you may just have to create a class like this:

.. code::

    class WestVirginiaIncludeEves(WestVirginia):
        west_virginia_include_christmas_eve = True
        west_virginia_include_nye = True

"""
from datetime import date

from ..registry_tools import iso_register
from .core import UnitedStates


@iso_register('US-WV')
class WestVirginia(UnitedStates):
    """West Virginia"""
    include_thanksgiving_friday = True
    include_election_day_even = True
    election_day_label = "Election Day / Susan B. Anthony Day"

    # West Virginia specific "half-holidays"
    west_virginia_include_christmas_eve = False
    west_virginia_include_nye = False
    FIXED_HOLIDAYS = UnitedStates.FIXED_HOLIDAYS + (
        (6, 20, "West Virgina Day"),
    )
    shift_exceptions = (
        (12, 24),
        (12, 31),
    )

    def get_fixed_holidays(self, year):
        days = super().get_fixed_holidays(year)
        if self.west_virginia_include_christmas_eve:
            days.append(
                (date(year, 12, 24), "Christmas Eve")
            )
        if self.west_virginia_include_nye:
            days.append(
                (date(year, 12, 31), "New Years Eve")
            )
        return days



from ..core import WesternCalendar
from ..registry_tools import iso_register


@iso_register('SK')
class Slovakia(WesternCalendar):
    'Slovakia'

    include_epiphany = True
    include_easter_monday = True
    include_good_friday = True
    include_all_saints = True
    include_christmas_eve = True
    include_boxing_day = True
    boxing_day_label = "St. Stephen's Day (The Second Christmas Day)"

    FIXED_HOLIDAYS = WesternCalendar.FIXED_HOLIDAYS + (
        (1, 1, "Day of the Establishment of the Slovak Republic"),
        (5, 1, "Labour Day"),
        (5, 8, "Liberation Day"),
        (7, 5, "Saints Cyril and Methodius Day"),
        (8, 29, "Slovak National Uprising anniversary"),
        (9, 1, "Day of the Constitution of the Slovak Republic"),
        (9, 15, "Day of Blessed Virgin Mary, patron saint of Slovakia"),
        (11, 17, "Struggle for Freedom and Democracy Day"),
    )



import warnings
from datetime import date
from ..core import MON, TUE, WED, THU, FRI, SAT
from ..registry_tools import iso_register

from .core import UnitedStates


@iso_register('US-IN')
class Indiana(UnitedStates):
    """Indiana"""
    include_good_friday = True
    include_thanksgiving_friday = True
    thanksgiving_friday_label = "Lincoln's Birthday"
    include_federal_presidents_day = False
    label_washington_birthday_december = "Washington's Birthday (Observed)"
    include_election_day_even = True
    election_day_label = "General Election Day"

    def get_washington_birthday_december(self, year):
        """
        Washington birthday observance
        Similar to Christmas Eve, but with special rules.
        It's only observed in Georgia.
        """
        warnings.warn(
            "Washington birthday rules for Indiana State are confusing. "
            "Use this calendar with care")
        christmas_day = date(year, 12, 25).weekday()
        if christmas_day == MON:
            day = date(year, 12, 26)  # TUE
        elif christmas_day == TUE:
            day = date(year, 12, 24)  # MON
        elif christmas_day == WED:
            day = date(year, 12, 26)  # TUE
        elif christmas_day == THU:
            day = date(year, 12, 26)  # FRI
        elif christmas_day == FRI:
            day = date(year, 12, 24)  # THU
        elif christmas_day == SAT:
            day = date(year, 12, 23)  # THU
        else:  # christmas_day == SUN:
            day = date(year, 12, 23)  # FRI
        return (day, self.label_washington_birthday_december)

    def get_primary_election_day(self, year):
        """
        Return the Primary Election Day

        FIXME: Wikipedia says it's a floating MON, but other sources say it's
        "the first Tuesday after the first Monday of May and every two years
        thereafter".
        """
        first_monday_may = self.get_nth_weekday_in_month(year, 5, MON)
        tuesday_after = self.get_nth_weekday_in_month(
            year, 5, TUE, start=first_monday_may)
        return (
            tuesday_after,
            "Primary Election Day"
        )

    def get_variable_days(self, year):
        days = super().get_variable_days(year)
        days.extend([
            self.get_washington_birthday_december(year),
        ])
        if (year % 2) == 0:
            days.append(self.get_primary_election_day(year))
        return days



from datetime import timedelta

from ..core import WesternCalendar
from ..registry_tools import iso_register


@iso_register('PA')
class Panama(WesternCalendar):
    "Panama"
    include_good_friday = True
    include_easter_saturday = True
    include_easter_sunday = True

    FIXED_HOLIDAYS = WesternCalendar.FIXED_HOLIDAYS + (
        (1, 9, "Martyrs' Day"),
        (5, 1, "Labour Day"),
        (11, 3, "Independence Day"),
        (11, 5, "Colon Day"),
        (11, 10, "Shout in Villa de los Santos"),
        (11, 28, "Independence from Spain"),
        (12, 8, "Mothers' Day"),
    )

    def get_variable_days(self, year):
        days = super().get_variable_days(year)
        days.append(
            (self.get_ash_wednesday(year) - timedelta(days=1), "Carnival")
        )
        return days



from ..core import WesternCalendar
from ..registry_tools import iso_register


@iso_register('PL')
class Poland(WesternCalendar):
    'Poland'

    FIXED_HOLIDAYS = WesternCalendar.FIXED_HOLIDAYS + (
        (1, 6, 'Trzech Kroli'),
        (5, 1, 'Labour Day'),
        (5, 3, 'Constitution Day'),
        (11, 11, 'Independence Day'),
    )
    include_easter_sunday = True
    include_easter_monday = True
    include_whit_sunday = True
    whit_sunday_label = "Pentecost Sunday"
    include_corpus_christi = True
    include_assumption = True
    include_all_saints = True
    include_boxing_day = True



from ..core import WesternCalendar
from ..registry_tools import iso_register


@iso_register('NO')
class Norway(WesternCalendar):
    'Norway'

    include_holy_thursday = True
    include_good_friday = True
    include_easter_sunday = True
    include_easter_monday = True
    include_ascension = True
    include_whit_monday = True
    include_whit_sunday = True
    include_boxing_day = True
    boxing_day_label = "St Stephen's Day"

    FIXED_HOLIDAYS = WesternCalendar.FIXED_HOLIDAYS + (
        (5, 1, "Labour Day"),
        (5, 17, "Constitution Day"),
    )



from datetime import date
from ..core import WesternCalendar, FRI, SAT
from ..registry_tools import iso_register


@iso_register('FI')
class Finland(WesternCalendar):
    'Finland'

    include_epiphany = True
    include_good_friday = True
    include_easter_sunday = True
    include_easter_monday = True
    include_ascension = True
    include_whit_sunday = True
    whit_sunday_label = 'Pentecost'
    include_christmas_eve = True
    include_boxing_day = True
    boxing_day_label = "St. Stephen's Day"

    FIXED_HOLIDAYS = WesternCalendar.FIXED_HOLIDAYS + (
        (5, 1, "Labour Day"),
        (12, 6, "Independence Day"),
    )

    def get_midsummer_eve(self, year):
        date_eve = Finland.get_nth_weekday_in_month(
            year, 6, FRI, start=date(year, 6, 19))
        return date_eve

    def get_midsummer_day(self, year):
        date_eve = Finland.get_nth_weekday_in_month(
            year, 6, SAT, start=date(year, 6, 20))
        return date_eve

    def get_variable_all_saints(self, year):
        all_saints = date(year, 10, 31)
        if all_saints.weekday() != SAT:
            all_saints = Finland.get_nth_weekday_in_month(
                year, 11, SAT)
        return all_saints

    def get_variable_days(self, year):
        days = super().get_variable_days(year)
        days.append((self.get_midsummer_eve(year), "Midsummer's Eve"))
        days.append((self.get_midsummer_day(year), "Midsummer's Day"))
        days.append((self.get_variable_all_saints(year), "All Saints"))
        return days



from .core import UnitedStates
from ..registry_tools import iso_register
from ..core import MON


@iso_register('US-IL')
class Illinois(UnitedStates):
    """Illinois"""
    include_thanksgiving_friday = True
    include_lincoln_birthday = True
    include_election_day_even = True


class ChicagoIllinois(Illinois):
    "Chicago, Illinois"
    include_thanksgiving_friday = False

    def get_pulaski_day(self, year):
        """
        Return Casimir Pulaski Day.

        Defined on the first MON of March.
        ref: https://en.wikipedia.org/wiki/Casimir_Pulaski_Day
        """
        day = self.get_nth_weekday_in_month(year, 3, MON)
        return (
            day,
            "Casimir Pulaski Day"
        )

    def get_variable_days(self, year):
        days = super().get_variable_days(year)
        days.append(self.get_pulaski_day(year))
        return days



from datetime import timedelta
from copy import copy

from ..core import WesternCalendar, SUN, MON
from ..registry_tools import iso_register


@iso_register("BB")
class Barbados(WesternCalendar):
    "Barbados"

    include_good_friday = True
    include_easter_sunday = True
    include_easter_monday = True
    include_whit_monday = True
    include_boxing_day = True

    # All holiday are shifted if on a Sunday
    FIXED_HOLIDAYS = WesternCalendar.FIXED_HOLIDAYS + (
        (1, 21, "Errol Barrow Day"),
        (4, 28, "National Heroes Day"),
        (5, 1, "Labour Day"),
        (8, 1, "Emancipation Day"),
        (11, 30, "Independance Day"),
    )

    def get_kadooment_day(self, year):
        """
        First Monday of August.
        """
        return (Barbados.get_nth_weekday_in_month(year, 8, MON),
                "Kadooment Day")

    def get_variable_days(self, year):
        """
        Return variable holidays of the Barbados calendar.
        """
        days = super().get_variable_days(year)
        days.append(self.get_kadooment_day(year))
        return days

    def get_fixed_holidays(self, year):
        """
        Return fixed holidays of the Barbados calendar.

        A shift day is appended if a fixed holiday happens on SUN.
        """
        days = super().get_fixed_holidays(year)
        days_to_shift = copy(days)
        for day, label in days_to_shift:
            if day.weekday() == SUN:
                days.append(
                    (day + timedelta(days=1), "%s %s" % (label, "(shifted)"))
                )
        return days



from ..core import WesternCalendar
from ..registry_tools import iso_register


@iso_register('ST')
class SaoTomeAndPrincipe(WesternCalendar):
    "São Tomé and Príncipe"
    FIXED_HOLIDAYS = WesternCalendar.FIXED_HOLIDAYS + (
        (2, 3, "Martyr's Day"),
        (5, 1, "Labour Day"),
        (7, 12, "Independence Day"),
        (9, 6, "Armed Forces Day"),
        (9, 30, "Agricultural Reform Day"),
        (12, 21, "São Tomé Day"),
    )
    include_all_saints = True
